<?php

/**
 * EXHIBIT A. Common Public Attribution License Version 1.0
 * The contents of this file are subject to the Common Public Attribution License Version 1.0 (the “License”);
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.oxwall.org/license.php. The License is based on the Mozilla Public License Version 1.1
 * but Sections 14 and 15 have been added to cover use of software over a computer network and provide for
 * limited attribution for the Original Developer. In addition, Exhibit A has been modified to be consistent
 * with Exhibit B. Software distributed under the License is distributed on an “AS IS” basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the specific language
 * governing rights and limitations under the License. The Original Code is Oxwall software.
 * The Initial Developer of the Original Code is Skalfa LLC (http://www.skalfa.com/).
 * All portions of the code written by Skalfa LLC are Copyright (c) 2009. All Rights Reserved.

 * EXHIBIT B. Attribution Information
 * Attribution Copyright Notice: Copyright 2009 Skalfa LLC. All rights reserved.
 * Attribution Phrase (not exceeding 10 words): Powered by Oxwall community software
 * Attribution URL: http://www.oxwall.org/
 * Graphic Image as provided in the Covered Code.
 * Display of Attribution Information is required in Larger Works which are defined in the CPAL as a work
 * which combines Covered Code or portions thereof with code not governed by the terms of the CPAL.
 */

/**
 * Base class for HTML document object,
 * handles rendered output context (html document includes, headers, styles, etc).
 * 
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow.ow_core
 * @since 1.0
 */
class OW_HtmlDocument extends OW_Document
{
    const META_EXPIRES = 'expires';
    const META_REFRESH = 'refresh';
    const META_AUTHOR = 'author';
    const META_GENERATOR = 'generator';
    const META_COPYRIGHT = 'copyright';
    const META_ROBOTS = 'robots';

    const META_DOCUMENT_STATE = 'document-state';
    const META_URL = 'url';
    const META_RESOURCE_TYPE = 'resource-type';
    const META_PICS_LABEL = 'pics-label';
    const META_REPLY_TO = 'reply-to';

    /**
     * Included stylesheet file urls.
     *
     * @var array
     */
    protected $styleSheets = array('added' => array());
    /**
     * Appended style declarations.
     *
     * @var string
     */
    protected $styleDeclarations = array('hash' => array());
    /**
     * Included javascript files.
     *
     * @var array
     */
    protected $javaScripts = array('added' => array());
    /**
     * Appended javascript code.
     *
     * @var string
     */
    protected $javaScriptDeclarations = array();
    /**
     * Javascript code added before script file includes.
     * @var <type>
     */
    protected $preIncludeJavaScriptDeclarations = array();
    /**
     * Appended onload javascript.
     * 
     * @var array
     */
    protected $onloadJavaScript = array('hash' => array());
    /**
     * Added head area links.
     *
     * @var array
     */
    private $links = array();
    /**
     * Added meta tags.
     *
     * @var array
     */
    private $meta = array();
    /**
     * Custom head info.
     * 
     * @var array
     */
    private $customHeadInfo = array();
    /**
     * Document master page.
     *
     * @var OW_MasterPage
     */
    private $masterPage;
    /**
     * Content area html code.
     *
     * @var string
     */
    private $body = '';
    /**
     * HTML code to be appended after document was rendered.
     *
     * @var string
     */
    private $appendCode = '';
    /**
     * HTML code to be prepended after document was rendered. 
     * 
     * @var string
     */
    private $prependCode = '';
    /**
     * Document keywords for meta tags.
     *
     * @var string
     */
    private $keywords;
    /**
     * Document heading.
     * 
     * @var string
     */
    private $heading;
    /**
     * Document heading icon class.
     *
     * @var string
     */
    private $headingIconClass;

    /**
     * Constructor.
     *
     * @param array $params
     */
    public function __construct()
    {
        $this->setType(OW_Document::HTML);
    }

    /**
     * Returns document heading icon class.
     *
     * @return string
     */
    public function getHeadingIconClass()
    {
        return $this->headingIconClass;
    }

    /**
     * Sets document heading icon class.
     *
     * @param string $headingIcon
     */
    public function setHeadingIconClass( $headingIconClass )
    {
        $this->headingIconClass = $headingIconClass;
    }

    /**
     * Sets document heading.
     * 
     * @param string $heading
     */
    public function setHeading( $heading )
    {
        $this->heading = $heading;
    }

    /**
     * @return string $heading
     */
    public function getHeading()
    {
        return $this->heading;
    }

    /**
     * Returns HTML document keywords.
     * 
     * @return string
     */
    public function getKeywords()
    {
        return $this->keywords;
    }

    /**
     * Sets HTML document keywords.
     * 
     * @param mixed $keywords
     * @return OW_HtmlDocument
     */
    public function setKeywords( $keywords )
    {
        if ( is_array($keywords) )
        {
            $keywords = implode(',', $keywords);
        }

        $this->keywords = $keywords;

        return $this;
    }

    /**
     * Sets html document body code.
     *
     * @param string $code
     * @return OW_HtmlDocument
     */
    public function setBody( $code )
    {
        $this->body = $code;

        return $this;
    }

    /**
     * Returns html document body.
     *
     * @return string
     */
    public function getBody()
    {
        return $this->body;
    }

    /**
     * Sets master page in html document.
     *
     * @param OW_MasterPage $masterPage
     * @return OW_HtmlDocument
     */
    public function setMasterPage( OW_MasterPage $masterPage )
    {
        $this->masterPage = $masterPage;
    }

    /**
     * Returns document master page.
     *
     * @return OW_MasterPage
     */
    public function getMasterPage()
    {
        return $this->masterPage;
    }

    /**
     * Adds stylesheet file to document.
     *
     * @param string $url
     * @return OW_HtmlDocument
     */
    public function addStyleSheet( $url, $media = 'all', $priority = null )
    {
        $url = trim($url);

        $media = trim($media);

        if ( in_array($url, $this->styleSheets['added']) )
        {
            return $this;
        }

        $priority = ($priority === null) ? 1000 : (int) $priority;

        $this->styleSheets['added'][] = $url;

        $this->styleSheets[$priority][$media][] = $url;

        return $this;
    }

    /**
     * Adds head style declarations to document.
     *
     * @param string $style
     * @return OW_HtmlDocument
     */
    public function addStyleDeclaration( $style, $media = 'all', $priority = null )
    {
        $media = trim(mb_strtolower($media));

        $styleHash = md5($style);

        if ( in_array($styleHash, $this->styleDeclarations['hash']) )
        {
            return $this;
        }

        $priority = ($priority === null) ? 1000 : (int) $priority;

        $this->styleDeclarations['hash'][] = $styleHash;

        $this->styleDeclarations[$priority][$media][] = $style;

        return $this;
    }

    /**
     * Adds javascript file to document.
     *
     * @param string $url
     * @param string $type
     * @return OW_HtmlDocument
     */
    public function addScript( $url, $type = "text/javascript", $priority = null )
    {
        $url = trim($url);

        if ( in_array($url, $this->javaScripts['added']) )
        {
            return $this;
        }

        $priority = ($priority === null) ? 1000 : (int) $priority;

        $this->javaScripts['added'][] = $url;

        $this->javaScripts[$priority][$type][] = $url;

        return $this;
    }

    /**
     * Adds head javascript code  to document.
     *
     * @param string $script
     * @param string $type
     * @return OW_HtmlDocument
     */
    public function addScriptDeclaration( $script, $type = "text/javascript", $priority = null )
    {
        $type = trim(mb_strtolower($type));

        $scriptHash = md5($script);

        if ( in_array($scriptHash, $this->javaScriptDeclarations) )
        {
            return $this;
        }

        $priority = ($priority === null) ? 1000 : (int) $priority;

        $this->javaScriptDeclarations[$priority][$type][] = $script;

        return $this;
    }

    /**
     * Adds onload javascript.
     * 
     * @param string $script
     */
    public function addOnloadScript( $script, $priority = null )
    {
        $scriptHash = md5($script);

        if ( in_array($scriptHash, $this->onloadJavaScript['hash']) )
        {
            return $this;
        }

        $priority = ($priority === null) ? 1000 : (int) $priority;

        $this->onloadJavaScript[$priority][] = $script;
    }

    /**
     * Adds head javascript code  to document before script file includes.
     *
     * @param string $script
     * @param string $type
     * @return OW_HtmlDocument
     */
    public function addScriptDeclarationBeforeIncludes( $script, $type = "text/javascript", $priority = null )
    {
        $type = trim(mb_strtolower($type));

        $scriptHash = md5($script);

        if ( in_array($scriptHash, $this->javaScriptDeclarations) )
        {
            return $this;
        }

        $priority = ($priority === null) ? 1000 : (int) $priority;

        $this->preIncludeJavaScriptDeclarations[$priority][$type][] = $script;

        return $this;
    }

    /**
     * Sets document favicon.
     *
     * @param string $url
     * @param string $type
     * @param string $relation
     * @return OW_HtmlDocument
     */
    public function setFavicon( $url )
    {
        $attributes = array('href' => trim($url), 'type' => 'image/x-icon', 'rel' => 'shortcut icon');

        $this->links[] = $attributes;

        return $this;
    }

    /**
     * Appends custom HTML to the rendered document.
     *
     * @param string $code
     * @return OW_HtmlDocument
     */
    public function appendBody( $code )
    {
        $this->appendCode .= $code;

        return $this;
    }

    /**
     * Prepends custom HTML code to the rendered document.
     * 
     * @param $code
     * @return OW_HtmlDocument
     */
    public function prependBody( $code )
    {
        $this->prependCode .= $code;

        return $this;
    }

    /**
     * Adds meta info to document.
     * You should also specify meta attribute name: `name` or `http-equiv`  
     *
     * @param string $name
     * @param string $value
     * @param string $attributeName
     * @return OW_HtmlDocument
     * @throws InvalidArgumentException
     */
    public function addMetaInfo( $name, $value, $attributeName = 'name' )
    {
        if ( $attributeName !== 'http-equiv' && $attributeName !== 'name' )
        {
            throw new InvalidArgumentException('Invalid meta attribute name was provided!');
        }

        $this->meta[$attributeName][$name] = $value;

        return $this;
    }

    /**
     * Adds cutom meta info.
     *
     * @param string $infoString
     */
    public function addCustomHeadInfo( $infoString )
    {
        $this->customHeadInfo[] = $infoString;
    }

    /*
     * i, we need to have modifications made to Skadate 7. We are a personal matching service and need to set our profiles in such a way that when a member logs in they are only able to VIEW and EDIT their own profile, NOT view or search for other profiles. Only us as the Administors will have access to member profiles and the search functions. I believe this is doable but need to know the costs involved. Cheers, Jo
     *
     *
     *
     * As I understand in your system guests and members are not allowed to view any pages except index/join/memberhome/edit profile and couple of other pages.
     * All the pages showing any info on other members should be hidden. Only site moders can view them. I can do it for $100.
     * If you need smth else please send me additional info. I'll send you details on implementation and price for work.
     
     * 
     *
     */

    /**
     * @return string
     */
    public function render()
    {
        $this->addMetaInfo('content-type', $this->getMime() . '; charset=' . $this->getCharset(), 'http-equiv');
        $this->addMetaInfo('keywords', $this->getKeywords());
        $this->addMetaInfo('description', $this->getDescription());

        $this->getMasterPage()->assign('content', $this->body);
        $this->getMasterPage()->assign('heading', $this->getHeading());
        $this->getMasterPage()->assign('heading_icon_class', $this->getHeadingIconClass());

        $masterPageOutput = $this->getMasterPage()->render();

        // META INFO		
        $metaInfo = '';

        foreach ( $this->meta['name'] as $name => $content )
        {
            $metaInfo .= '<meta name="' . $name . '" content="' . $content . '" />' . PHP_EOL;
        }

        foreach ( $this->meta['http-equiv'] as $name => $content )
        {
            $metaInfo .= '<meta http-equiv="' . $name . '" content="' . $content . '" />' . PHP_EOL;
        }

        $metaInfo .= implode('', $this->customHeadInfo);

        // CSS FILE INCLUDES
        $cssIncludeString = '';

        unset($this->styleSheets['added']);

        ksort($this->styleSheets);

        foreach ( $this->styleSheets as $priority => $scipts )
        {
            foreach ( $scipts as $media => $urls )
            {
                foreach ( $urls as $url )
                {
                    $cssIncludeString .= '<link rel="stylesheet" type="text/css" href="' . $url . '" ' . (($media === 'all') ? '' : 'media="' . $media . '"') . ' />' . PHP_EOL;
                }
            }
        }

        // CSS HEAD DECLARATIONS
        $cssHeadCode = '';

        unset($this->styleDeclarations['hash']);

        ksort($this->styleDeclarations);

        foreach ( $this->styleDeclarations as $priority => $mediaTypes )
        {
            foreach ( $mediaTypes as $media => $declarations )
            {
                foreach ( $declarations as $declaration )
                {
                    $cssHeadCode .= '<style' . ($media === 'all' ? '' : ' media="' . $media . '"') . '>' . PHP_EOL . $declaration . PHP_EOL . '</style>' . PHP_EOL;
                }
            }
        }


        // JS FILE INCLUDES
        $jsIncludeString = '';

        unset($this->javaScripts['added']);

        ksort($this->javaScripts);

        foreach ( $this->javaScripts as $priority => $types )
        {
            foreach ( $types as $type => $urls )
            {
                foreach ( $urls as $url )
                {
                    $jsIncludeString .= '<script type="' . $type . '" src="' . $url . '"></script>' . PHP_EOL;
                }
            }
        }

        // JS HEAD DECLARATIONS
        $jsHeadCode = '';

        unset($this->javaScriptDeclarations['hash']);

        ksort($this->javaScriptDeclarations);

        foreach ( $this->javaScriptDeclarations as $priority => $types )
        {
            foreach ( $types as $type => $declarations )
            {
                foreach ( $declarations as $declaration )
                {
                    $jsHeadCode .= '<script type="' . $type . '">' . PHP_EOL . $declaration . PHP_EOL . '</script>' . PHP_EOL;
                }
            }
        }

        // JS PRE INCLUDES HEAD DECLARATIONS
        $preIncludesjsHeadCode = '';

        unset($this->preIncludeJavaScriptDeclarations['hash']);

        ksort($this->preIncludeJavaScriptDeclarations);

        foreach ( $this->preIncludeJavaScriptDeclarations as $priority => $types )
        {
            foreach ( $types as $type => $declarations )
            {
                foreach ( $declarations as $declaration )
                {
                    $preIncludesjsHeadCode .= '<script type="' . $type . '">' . PHP_EOL . $declaration . PHP_EOL . '</script>' . PHP_EOL;
                }
            }
        }

        // ONLOAD JS
        $jsHeadCode .= '<script type="text/javascript">' . PHP_EOL . '$(function () {' . PHP_EOL;

        ksort($this->onloadJavaScript);

        foreach ( $this->onloadJavaScript as $priority => $scripts )
        {
            foreach ( $scripts as $script )
            {
                $jsHeadCode .= $script . PHP_EOL;
            }
        }

        $jsHeadCode .= PHP_EOL . '});' . PHP_EOL . '</script>';

        $assignArray = array(
            'title' => $this->getTitle(),
            'metaTags' => $metaInfo,
            'language' => $this->language,
            'direction' => $this->direction,
            'stylesheets' => $cssIncludeString,
            'cssCode' => $cssHeadCode,
            'jsIncludes' => $jsIncludeString,
            'jsCode' => $jsHeadCode,
            'preIncludesjsCode' => $preIncludesjsHeadCode,
            'pageBody' => $this->prependCode . $masterPageOutput . $this->appendCode);

        $renderer = OW_ViewRenderer::getInstance();

        $renderer->clearAssignedVars();

        $renderer->assignVars($assignArray);

        return $renderer->renderTemplate($this->getTemplate());
    }
}